import { ServerConnection } from '@jupyterlab/services';
import { Signal } from '@lumino/signaling';
export class FileSystemDrive {
    constructor() {
        this._isDisposed = false;
        this._fileChanged = new Signal(this);
        this._rootHandle = null;
    }
    get isDisposed() {
        return this._isDisposed;
    }
    dispose() {
        if (this.isDisposed) {
            return;
        }
        this._isDisposed = true;
        Signal.clearData(this);
    }
    get name() {
        return 'FileSystem';
    }
    get serverSettings() {
        return ServerConnection.makeSettings();
    }
    get fileChanged() {
        return this._fileChanged;
    }
    get rootHandle() {
        return this._rootHandle;
    }
    set rootHandle(handle) {
        this._rootHandle = handle;
    }
    async get(localPath, options) {
        const root = this._rootHandle;
        if (!root) {
            return {
                name: '',
                path: '',
                created: new Date().toISOString(),
                last_modified: new Date().toISOString(),
                format: 'json',
                content: null,
                writable: true,
                type: 'directory',
                mimetype: 'application/json'
            };
        }
        if (localPath) {
            const handle = await root.getFileHandle(localPath);
            const file = await handle.getFile();
            return {
                name: file.name,
                path: file.name,
                created: new Date(file.lastModified).toISOString(),
                last_modified: new Date(file.lastModified).toISOString(),
                format: 'text',
                content: await file.text(),
                writable: true,
                type: 'file',
                mimetype: 'text/plain'
            };
        }
        const content = [];
        for await (const value of root.values()) {
            if (value.kind === 'file') {
                const file = await value.getFile();
                content.push({
                    name: file.name,
                    path: file.name,
                    created: new Date(file.lastModified).toISOString(),
                    last_modified: new Date(file.lastModified).toISOString(),
                    format: 'text',
                    content: await file.text(),
                    writable: true,
                    type: 'file',
                    mimetype: 'text/plain'
                });
            }
            else {
                content.push({
                    name: value.name,
                    path: value.name,
                    created: new Date().toISOString(),
                    last_modified: new Date().toISOString(),
                    format: 'json',
                    content: null,
                    writable: true,
                    type: 'directory',
                    mimetype: 'application/json'
                });
            }
        }
        const date = new Date();
        return {
            name: 'root',
            path: '',
            last_modified: date.toISOString(),
            created: date.toISOString(),
            format: 'json',
            mimetype: 'application/json',
            content,
            size: undefined,
            writable: true,
            type: 'directory'
        };
    }
    getDownloadUrl(localPath) {
        throw new Error('Method not implemented.');
    }
    newUntitled(options) {
        throw new Error('Method not implemented.');
    }
    delete(localPath) {
        throw new Error('Method not implemented.');
    }
    rename(oldLocalPath, newLocalPath) {
        throw new Error('Method not implemented.');
    }
    async save(localPath, options) {
        const root = this._rootHandle;
        if (!root) {
            throw new Error('No root file handle');
        }
        const handle = await root.getFileHandle(localPath);
        const writable = await handle.createWritable({});
        const format = options?.format;
        const content = options?.content;
        if (format === 'json') {
            const data = JSON.stringify(content, null, 2);
            await writable.write(data);
        }
        else {
            await writable.write(content);
        }
        await writable.close();
        return this.get(localPath);
    }
    copy(localPath, toLocalDir) {
        throw new Error('Method not implemented.');
    }
    async createCheckpoint(localPath) {
        return {
            id: 'test',
            last_modified: new Date().toISOString()
        };
    }
    async listCheckpoints(localPath) {
        return [
            {
                id: 'test',
                last_modified: new Date().toISOString()
            }
        ];
    }
    restoreCheckpoint(localPath, checkpointID) {
        return Promise.resolve(void 0);
    }
    deleteCheckpoint(localPath, checkpointID) {
        return Promise.resolve(void 0);
    }
}
